package PScm;

use strict;
use warnings;
use PScm::Read;
use PScm::Env;
use PScm::Primitive;
use PScm::SpecialForm;
use PScm::Continuation;
use FileHandle;

require Exporter;

our @ISA    = qw(Exporter);
our @EXPORT = qw(ReadEvalPrint);

=head1 NAME

PScm - PScm interpreter written in Perl

=head1 SYNOPSIS

  use PScm;
  ReadEvalPrint($in_filehandle[, $out_filehandle]);

=head1 DESCRIPTION

Just messing about, A toy lisp interpreter.

=cut

sub ReadEvalPrint {
    my ($infh, $outfh) = @_;

    $outfh ||= new FileHandle(">-");
    my $reader = new PScm::Read($infh);
    trampoline(cont { repl($reader, $outfh) });
}

sub repl {
    my ($reader, $outfh) = @_;
    $reader->Read(
        cont {
            my ($expr) = @_;
            $expr->Eval(
                new PScm::Env(
                    let    => new PScm::SpecialForm::Let(),
                    '*'    => new PScm::Primitive::Multiply(),
                    '-'    => new PScm::Primitive::Subtract(),
                    if     => new PScm::SpecialForm::If(),
                    lambda => new PScm::SpecialForm::Lambda(),
                    letrec => new PScm::SpecialForm::LetRec(),
                    'let*' => new PScm::SpecialForm::LetStar(),
                    quote  => new PScm::SpecialForm::Quote(),
                    list   => new PScm::Primitive::List(),
                    car    => new PScm::Primitive::Car(),
                    cdr    => new PScm::Primitive::Cdr(),
                    cons   => new PScm::Primitive::Cons(),
                    eval   => new PScm::SpecialForm::Eval(),
                    macro  => new PScm::SpecialForm::Macro(),
                    'set!' => new PScm::SpecialForm::Set(),
                    begin  => new PScm::SpecialForm::Begin(),
                    'call/cc' =>
                      new PScm::SpecialForm::CallCC(),
                ),
                cont {
                    my ($result) = @_;
                    $result->Print(
                        $outfh,
                        cont {
                            repl($reader, $outfh);
                        }
                    )
                }
            )
        }
    )
}


sub trampoline {
    my ($cont) = @_;
    $cont = $cont->Bounce() while defined $cont;
}

sub Print {
    my ($expr, $outfh, $cont) = @_;
    print $outfh $expr->as_string, "\n";
    $cont->Cont($expr);
}

sub as_string { ref($_[0]); }

sub new { bless {}, $_[0] }

1;
